@@ -1,7 +1,7 @@
 /*
  * weechat-ruby.c - ruby plugin for WeeChat
  *
- * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  * Copyright (C) 2005-2007 Emmanuel Bouthenot <kolter@openics.org>
  *
  * This file is part of WeeChat, the extensible chat client.
@@ -274,7 +274,7 @@ weechat_ruby_print_exception (VALUE err)
     backtrace = rb_protect_funcall (err, rb_intern ("backtrace"),
                                     &ruby_error, 0, NULL);
 
-    message = rb_protect_funcall(err, rb_intern ("message"), &ruby_error, 0, NULL);
+    message = rb_protect_funcall (err, rb_intern ("message"), &ruby_error, 0, NULL);
     err_msg = StringValueCStr (message);
 
     err_class = NULL;
@@ -287,7 +287,7 @@ weechat_ruby_print_exception (VALUE err)
         err_class = StringValuePtr (class_name);
     }
 
-    if (strcmp (err_class, "SyntaxError") == 0)
+    if (err_class && (strcmp (err_class, "SyntaxError") == 0))
     {
         tmp3 = rb_inspect (err);
         weechat_printf (NULL,
@@ -437,8 +437,7 @@ weechat_ruby_output (VALUE self, VALUE str)
     }
     weechat_string_dyn_concat (ruby_buffer_output, ptr_msg, -1);
 
-    if (msg)
-        free (msg);
+    free (msg);
 
     return Qnil;
 }
@@ -762,8 +761,7 @@ weechat_ruby_unload (struct t_plugin_script *script)
                                        WEECHAT_SCRIPT_EXEC_INT,
                                        script->shutdown_func,
                                        0, NULL);
-        if (rc)
-            free (rc);
+        free (rc);
     }
 
     filename = strdup (script->filename);
@@ -781,8 +779,7 @@ weechat_ruby_unload (struct t_plugin_script *script)
 
     (void) weechat_hook_signal_send ("ruby_script_unloaded",
                                      WEECHAT_HOOK_SIGNAL_STRING, filename);
-    if (filename)
-        free (filename);
+    free (filename);
 }
 
 /*
@@ -875,13 +872,15 @@ weechat_ruby_eval (struct t_gui_buffer *buffer, int send_to_buffer_as_input,
 {
     void *func_argv[1], *result;
     char empty_arg[1] = { '\0' };
+    int old_ruby_quiet;
 
     if (!ruby_script_eval)
     {
+        old_ruby_quiet = ruby_quiet;
         ruby_quiet = 1;
         ruby_script_eval = weechat_ruby_load (WEECHAT_SCRIPT_EVAL_NAME,
                                               RUBY_EVAL_SCRIPT);
-        ruby_quiet = 0;
+        ruby_quiet = old_ruby_quiet;
         if (!ruby_script_eval)
             return 0;
     }
@@ -899,8 +898,7 @@ weechat_ruby_eval (struct t_gui_buffer *buffer, int send_to_buffer_as_input,
                                 "script_ruby_eval",
                                 "s", func_argv);
     /* result is ignored */
-    if (result)
-        free (result);
+    free (result);
 
     weechat_ruby_output_flush ();
 
@@ -911,9 +909,10 @@ weechat_ruby_eval (struct t_gui_buffer *buffer, int send_to_buffer_as_input,
 
     if (!weechat_config_boolean (ruby_config_look_eval_keep_context))
     {
+        old_ruby_quiet = ruby_quiet;
         ruby_quiet = 1;
         weechat_ruby_unload (ruby_script_eval);
-        ruby_quiet = 0;
+        ruby_quiet = old_ruby_quiet;
         ruby_script_eval = NULL;
     }
 
@@ -930,7 +929,7 @@ weechat_ruby_command_cb (const void *pointer, void *data,
                          int argc, char **argv, char **argv_eol)
 {
     char *ptr_name, *ptr_code, *path_script;
-    int i, send_to_buffer_as_input, exec_commands;
+    int i, send_to_buffer_as_input, exec_commands, old_ruby_quiet;
 
     /* make C compiler happy */
     (void) pointer;
@@ -989,6 +988,7 @@ weechat_ruby_command_cb (const void *pointer, void *data,
                  || (weechat_strcmp (argv[1], "reload") == 0)
                  || (weechat_strcmp (argv[1], "unload") == 0))
         {
+            old_ruby_quiet = ruby_quiet;
             ptr_name = argv_eol[2];
             if (strncmp (ptr_name, "-q ", 3) == 0)
             {
@@ -1003,11 +1003,10 @@ weechat_ruby_command_cb (const void *pointer, void *data,
             {
                 /* load ruby script */
                 path_script = plugin_script_search_path (weechat_ruby_plugin,
-                                                         ptr_name);
+                                                         ptr_name, 1);
                 weechat_ruby_load ((path_script) ? path_script : ptr_name,
                                    NULL);
-                if (path_script)
-                    free (path_script);
+                free (path_script);
             }
             else if (weechat_strcmp (argv[1], "reload") == 0)
             {
@@ -1019,7 +1018,7 @@ weechat_ruby_command_cb (const void *pointer, void *data,
                 /* unload ruby script */
                 weechat_ruby_unload_name (ptr_name);
             }
-            ruby_quiet = 0;
+            ruby_quiet = old_ruby_quiet;
         }
         else if (weechat_strcmp (argv[1], "eval") == 0)
         {
@@ -1265,8 +1264,9 @@ weechat_ruby_signal_script_action_cb (const void *pointer, void *data,
 int
 weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 {
-    int ruby_error;
+    int ruby_error, old_ruby_quiet;
     VALUE err;
+    char* process_options_argv[] = { "ruby", "-enil", NULL };
     char *weechat_ruby_code = {
         "$stdout = WeechatOutputs\n"
         "$stderr = WeechatOutputs\n"
@@ -1336,6 +1336,11 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 
     weechat_ruby_plugin = plugin;
 
+    ruby_quiet = 0;
+    ruby_eval_mode = 0;
+    ruby_eval_send_input = 0;
+    ruby_eval_exec_commands = 0;
+
     /* set interpreter name and version */
     weechat_hashtable_set (plugin->variables, "interpreter_name",
                            plugin->name);
@@ -1360,6 +1365,8 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 
     ruby_init ();
 
+    ruby_process_options (2, process_options_argv);
+
     /* redirect stdin and stdout */
     ruby_mWeechatOutputs = rb_define_module ("WeechatOutputs");
     rb_define_singleton_method (ruby_mWeechatOutputs, "write",
@@ -1404,11 +1411,13 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
     ruby_data.callback_signal_debug_dump = &weechat_ruby_signal_debug_dump_cb;
     ruby_data.callback_signal_script_action = &weechat_ruby_signal_script_action_cb;
     ruby_data.callback_load_file = &weechat_ruby_load_cb;
+    ruby_data.init_before_autoload = NULL;
     ruby_data.unload_all = &weechat_ruby_unload_all;
 
+    old_ruby_quiet = ruby_quiet;
     ruby_quiet = 1;
     plugin_script_init (weechat_ruby_plugin, &ruby_data);
-    ruby_quiet = 0;
+    ruby_quiet = old_ruby_quiet;
 
     plugin_script_display_short_list (weechat_ruby_plugin,
                                       ruby_scripts);
@@ -1424,7 +1433,10 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 int
 weechat_plugin_end (struct t_weechat_plugin *plugin)
 {
+    int old_ruby_quiet;
+
     /* unload all scripts */
+    old_ruby_quiet = ruby_quiet;
     ruby_quiet = 1;
     if (ruby_script_eval)
     {
@@ -1432,19 +1444,29 @@ weechat_plugin_end (struct t_weechat_plugin *plugin)
         ruby_script_eval = NULL;
     }
     plugin_script_end (plugin, &ruby_data);
-    ruby_quiet = 0;
+    ruby_quiet = old_ruby_quiet;
 
     ruby_cleanup (0);
     signal (SIGCHLD, SIG_DFL);
 
     /* free some data */
     if (ruby_action_install_list)
+    {
         free (ruby_action_install_list);
+        ruby_action_install_list = NULL;
+    }
     if (ruby_action_remove_list)
+    {
         free (ruby_action_remove_list);
+        ruby_action_remove_list = NULL;
+    }
     if (ruby_action_autoload_list)
+    {
         free (ruby_action_autoload_list);
+        ruby_action_autoload_list = NULL;
+    }
     weechat_string_dyn_free (ruby_buffer_output, 1);
+    ruby_buffer_output = NULL;
 
     return WEECHAT_RC_OK;
 }
